'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _MAX_DISCRETE_VALUES;

var _reselect = require('reselect');

var _urlRegex = require('url-regex');

var _urlRegex2 = _interopRequireDefault(_urlRegex);

var _url = require('url');

var _url2 = _interopRequireDefault(_url);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _isPlainObject = require('is-plain-object');

var _isPlainObject2 = _interopRequireDefault(_isPlainObject);

var _constants = require('../constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var urlRegex = (0, _urlRegex2.default)({ exact: true });

// Max number of unique text values for a field to still be a filterable discrete field
var MAX_DISCRETE_VALUES = (_MAX_DISCRETE_VALUES = {}, (0, _defineProperty3.default)(_MAX_DISCRETE_VALUES, _constants.FIELD_TYPES.STRING, 15), (0, _defineProperty3.default)(_MAX_DISCRETE_VALUES, _constants.FIELD_TYPES.NUMBER, 5), _MAX_DISCRETE_VALUES);

var imageExts = ['jpg', 'tif', 'jpeg', 'png', 'tiff', 'webp'];
var videoExts = ['mov', 'mp4', 'avi', 'webm'];
var audioExts = ['3gpp', 'wav', 'wma', 'mp3', 'm4a', 'aiff', 'ogg'];
var mediaExts = imageExts.concat(videoExts, audioExts);

var filterableTypes = [_constants.FIELD_TYPES.DATE, _constants.FIELD_TYPES.STRING, _constants.FIELD_TYPES.NUMBER, _constants.FIELD_TYPES.BOOLEAN, _constants.FIELD_TYPES.ARRAY, _constants.FIELD_TYPES.STRING_OR_ARRAY, _constants.FIELD_TYPES.NUMBER_OR_ARRAY];

function wc(s) {
  return s.split(/ |_/).length;
}

function isFilterable(type) {
  return filterableTypes.indexOf(type) > -1;
}

/**
 * Reducer that computes running mean, variance, min and max
 * Adapted from http://www.johndcook.com/blog/standard_deviation/
 * @param {Object} p The previous value for the analysis
 * @param {Number} x New value to be included in analysis
 * @param {Number} i Index of the current element being processed
 * @return {Object} New analysis including `x`
 */
function statReduce() {
  var p = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : { mean: NaN, vari: NaN, min: +Infinity, max: -Infinity };
  var x = arguments[1];
  var i = arguments[2];

  p.mean = isNaN(p.mean) ? 0 : p.mean;
  var mean = p.mean + (x - p.mean) / (i + 1);
  x = x instanceof Date ? +x : x;
  return {
    mean: mean,
    min: x < p.min ? x : p.min,
    max: x > p.max ? x : p.max,
    vari: i < 1 ? 0 : (p.vari * i + (x - p.mean) * (x - mean)) / (i + 1)
  };
}

function isMediaField(fieldType) {
  return [_constants.FIELD_TYPES.VIDEO, _constants.FIELD_TYPES.IMAGE, _constants.FIELD_TYPES.MEDIA].indexOf(fieldType) > -1;
}

function isStringOrArray(fieldType) {
  return [_constants.FIELD_TYPES.STRING, _constants.FIELD_TYPES.ARRAY, _constants.FIELD_TYPES.STRING_OR_ARRAY].indexOf(fieldType) > -1;
}

function isNumberOrArray(fieldType) {
  return [_constants.FIELD_TYPES.NUMBER, _constants.FIELD_TYPES.ARRAY, _constants.FIELD_TYPES.NUMBER_OR_ARRAY].indexOf(fieldType) > -1;
}

/**
 * Reducer that returns 'mixed' if values are not all the same,
 * or 'media' if field is a mixture of image and video files
 * @param {any} p Previous value
 * @param {any} v Current value
 * @return {any} `v` or `mixed`
 */
function typeReduce(p, v) {
  if (!p || v === p) return v;
  if (isMediaField(p) && isMediaField(v)) {
    return _constants.FIELD_TYPES.MEDIA;
  } else if (isStringOrArray(p) && isStringOrArray(v)) {
    return _constants.FIELD_TYPES.STRING_OR_ARRAY;
  } else if (isNumberOrArray(p) && isNumberOrArray(v)) {
    return _constants.FIELD_TYPES.NUMBER_OR_ARRAY;
  } else {
    return _constants.FIELD_TYPES.MIXED;
  }
}

function valuesReduce() {
  var p = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var v = arguments[1];

  v = Array.isArray(v) ? v : [v];
  v.forEach(function (w) {
    p[w] = typeof p[w] === 'undefined' ? 1 : p[w] + 1;
  });
  return p;
}

function getFilterType(f) {
  var keyCount = f.values && (0, _keys2.default)(f.values).length;
  if (!isFilterable(f.type)) return;
  // No point in having a filter for only one value
  if (keyCount === 1) return;
  switch (f.type) {
    case _constants.FIELD_TYPES.DATE:
      return _constants.FILTER_TYPES.DATE;
    case _constants.FIELD_TYPES.NUMBER:
      if (keyCount <= MAX_DISCRETE_VALUES[_constants.FIELD_TYPES.NUMBER]) {
        return _constants.FILTER_TYPES.DISCRETE;
      } else {
        f.values = undefined;
        return _constants.FILTER_TYPES.RANGE;
      }
    case _constants.FIELD_TYPES.BOOLEAN:
      return _constants.FILTER_TYPES.DISCRETE;
    case _constants.FIELD_TYPES.STRING:
    case _constants.FIELD_TYPES.ARRAY:
    case _constants.FIELD_TYPES.STRING_OR_ARRAY:
    case _constants.FIELD_TYPES.NUMBER_OR_ARRAY:
      if (keyCount <= MAX_DISCRETE_VALUES[_constants.FIELD_TYPES.STRING]) {
        return _constants.FILTER_TYPES.DISCRETE;
      } else {
        f.values = undefined;
        return _constants.FILTER_TYPES.TEXT;
      }
  }
}

var potentialUUIDTypes = [_constants.FIELD_TYPES.STRING, _constants.FIELD_TYPES.NUMBER];

/**
 * Guess if a field can be used as a UUID: There are as many different values
 * as there are features, and wordcount === 1 - this is to avoid text fields
 * been classified as UUID fields.
 * @param {object} f A field object with analysis props
 * @param {array} features Array of features
 * @return {Boolean} [description]
 */
function isUnique(f, features) {
  if (potentialUUIDTypes.indexOf(f.type) < 0) return;
  var keyCount = f.values && (0, _keys2.default)(f.values).length;
  return features.length === keyCount && f.wordStats.max === 1;
}

/**
 * Returns the type of a value, guessing types `date`, `link`, `image`, `video`
 * @param {any} v Type to be evaluated
 * @return {string} One of `string`, `number`, `bool`, `date`, `link`, `image`, `video`
 */
function getType(v) {
  if (isDate(v)) return _constants.FIELD_TYPES.DATE;
  if (typeof v === 'string') {
    if (urlRegex.test(v)) {
      var pathname = _url2.default.parse(v).pathname;
      var _ext = _path2.default.extname(pathname).slice(1);
      if (imageExts.indexOf(_ext) > -1) return _constants.FIELD_TYPES.IMAGE;
      if (videoExts.indexOf(_ext) > -1) return _constants.FIELD_TYPES.VIDEO;
      if (audioExts.indexOf(_ext) > -1) return _constants.FIELD_TYPES.AUDIO;
      return _constants.FIELD_TYPES.LINK;
    }
    var ext = _path2.default.extname(v).slice(1);
    if (v.split('/').length === 1 && mediaExts.indexOf(ext) > -1) return _constants.FIELD_TYPES.FILENAME;
  }
  if (Array.isArray(v)) return _constants.FIELD_TYPES.ARRAY;
  return typeof v === 'undefined' ? 'undefined' : (0, _typeof3.default)(v);
}

function isDate(v) {
  if (v instanceof Date) return true;
  return new Date(v) !== 'Invalid Date' && !isNaN(new Date(v));
}

/**
 * Guess if a field is a UUID: if it has a length greater than
 * 30, no variance in length, and is only one word.
**/
function isUUIDField(f) {
  if (!f.isUnique) return;
  if (f.type !== _constants.FIELD_TYPES.STRING) return;
  return f.lengthStats.mean > 30 && f.lengthStats.vari === 0 && f.wordStats.max === 1;
}

/**
 * Analyzes the fields of features in a featureCollection and guesses the
 *   field type and what type of filter to use: `discrete`, `number`,
 *   `date` (subtype of continuous), or `text` (and field that has more than
 *   `maxTextValues` discrete values). Number fields with <= `maxNumberCount`
 *   different values are considered discrete.
 * @param {object} featureCollection GeoJson FeatureCollection
 * @return {object} An object with a key for each unique field name in the
 *   FeatureCollection with properties `type` of filter to use, a count for
 *   each discrete option, or a min/max for continuous fields
 */
var getFieldAnalysis = (0, _reselect.createSelector)(function (state) {
  return state.features;
}, function analyzeFields(features) {
  var analysis = {};
  var idFieldValues = {};
  // Iterate over every feature in the FeatureCollection
  for (var i = 0; i < features.length; i++) {
    if (features[i].id) idFieldValues = valuesReduce(idFieldValues, features[i].id);
    // For each feature, iterate over its properties
    traverse(features[i].properties, i);
  }

  function traverse(obj, i) {
    var prefix = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '';

    var keys = (0, _keys2.default)(obj);
    for (var j = 0; j < keys.length; j++) {
      var value = obj[keys[j]];
      var fieldname = prefix + keys[j];
      if ((0, _isPlainObject2.default)(value)) {
        traverse(value, i, fieldname + '.');
        continue;
      }
      var field = analysis[fieldname] = analysis[fieldname] || { fieldname: fieldname };
      var thisType = getType(value);
      field.type = typeReduce(field.type, thisType);
      field.count = (field.count || 0) + 1;
      if (!isFilterable(field.type)) continue;
      if (thisType === _constants.FIELD_TYPES.STRING) {
        field.wordStats = statReduce(field.wordStats, wc(value), i);
        field.lengthStats = statReduce(field.lengthStats, value.length, i);
      }
      if (thisType === _constants.FIELD_TYPES.ARRAY) {
        field.maxArrayLength = Math.max(field.maxArrayLength || 1, value.length);
      } else if (thisType === _constants.FIELD_TYPES.NUMBER) {
        field.valueStats = statReduce(field.valueStats, value, i);
      } else if (thisType === _constants.FIELD_TYPES.DATE) {
        field.valueStats = statReduce(field.valueStats, new Date(value), i);
      }
      field.values = valuesReduce(field.values, value);
    }
  }

  for (var fieldname in analysis) {
    var field = analysis[fieldname];
    field.isUnique = isUnique(field, features);
    if (isUUIDField(field)) field.type = _constants.FIELD_TYPES.UUID;
    field.filterType = getFilterType(field);
  }
  var isIdFieldUnique = (0, _keys2.default)(idFieldValues).length === features.length;
  // TODO: this could cause an edge-case (extremely unlikely) bug if a feature
  // has an existing property called `__validGeoJsonIdField`.
  // Should restructure analysis object
  analysis.__validGeoJsonIdField = isIdFieldUnique;
  return analysis;
});

exports.default = getFieldAnalysis;